--[[
    File: tuf_options.lua
    Author: Tuck
    Description: This file contains all funtions that handle options of TuckUF.
]]

local core = TuckUF
local TRACE = function(msg) ChatFrame1:AddMessage(msg) end

local L = LibStub("AceLocale-3.0"):GetLocale("TuckUF", true)
local config = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")
local playerclass = select(2, UnitClass("player"))

local default_border_size = 4

local defaults = {
    profile = {
        -- click casting
        click_action = "target",
        alt_click_action = "assist",
        shift_click_action = "",
        ctrl_click_action = "",
        
        -- player
        player_enabled = true,
        player_width = 200,
        player_height = 46,
        player_border = default_border_size,
        player_hp_text = true,
        player_hp_percent = false,
        player_mana_text = true,
        player_mana_bar = "big",
        
        player_buffs_enabled = true,
        player_buffs_rows = 2,
        player_buffs_columns = 12,
        player_buffs_icon_size = 12,
        player_buffs_anchor = "BOTTOMLEFT",
        
        player_buffs2_enabled = true,
        player_buffs2_rows = 3,
        player_buffs2_columns = 1,
        player_buffs2_icon_size = 24,
        player_buffs2_anchor = "LEFTTOP",
        player_buffs2_filter_weak = true,
        player_buffs2_sort_mode = "max_power",
        player_buffs2_offset_x = -8,
        
        player_debuffs_enabled = true,
        player_debuffs_rows = 2,
        player_debuffs_columns = 2,
        player_debuffs_icon_size = 28,
        player_debuffs_anchor = "RIGHT",
        
        player_debuffs2_enabled = true,
        player_debuffs2_rows = 1,
        player_debuffs2_columns = 2,
        player_debuffs2_icon_size = 38,
        player_debuffs2_transparency = 0,
        player_debuffs2_anchor = "INSIDERIGHT",
        player_debuffs2_filter_weak = true,
        player_debuffs2_group_mode = "group",
        player_debuffs2_sort_mode = "max_power",
        player_debuffs2_offset_x = -default_border_size,
        
        -- pet
        pet_enabled = true,
        pet_width = 200,
        pet_height = 26,
        pet_border = default_border_size,
        pet_mana_bar = "small",
        
        pet_buffs_enabled = true,
        pet_buffs_rows = 1,
        pet_buffs_columns = 11,
        pet_buffs_icon_size = 12,
        pet_buffs_anchor = "BOTTOMLEFT",
        
        pet_debuffs_enabled = true,
        pet_debuffs_rows = 2,
        pet_debuffs_columns = 2,
        pet_debuffs_icon_size = 20,
        pet_debuffs_anchor = "RIGHT",
        
        -- target
        target_enabled = true,
        target_width = 200,
        target_height = 46,
        target_border = default_border_size,
        target_mana_bar = "big",
        
        target_buffs_enabled = true,
        target_buffs_rows = 2,
        target_buffs_columns = 12,
        target_buffs_icon_size = 12,
        target_buffs_anchor = "BOTTOMLEFT",
        
        target_debuffs_enabled = true,
        target_debuffs_rows = 2,
        target_debuffs_columns = 10,
        target_debuffs_icon_size = 28,
        target_debuffs_anchor = "BOTTOMLEFT",
        target_debuffs_offset_y = -24,
        
        target_buffs2_enabled = true,
        target_buffs2_rows = 1,
        target_buffs2_columns = 4,
        target_buffs2_icon_size = 32,
        target_buffs2_anchor = "INSIDELEFT",
        target_buffs2_group_mode = "none",
        target_buffs2_sort_mode = "max_power",
        target_buffs2_filter_weak = true,
        target_buffs2_filter_normal = true,
        target_buffs2_offset_x = default_border_size,
        
        target_debuffs2_enabled = true,
        target_debuffs2_rows = 1,
        target_debuffs2_columns = 2,
        target_debuffs2_icon_size = 38,
        target_debuffs2_transparency = 0,
        target_debuffs2_anchor = "INSIDERIGHT",
        target_debuffs2_filter_weak = true,
        target_debuffs2_group_mode = "group",
        target_debuffs2_sort_mode = "max_power",
        target_debuffs2_offset_x = -default_border_size,
        
        -- ToT and ToToT
        tot_enabled = true,
        tot_width = 80,
        tot_height = 30,
        tot_border = default_border_size,
        tot_hp_text = false,
        tot_mana_text = false,
        tot_mana_bar = "small",
                
        totot_enabled = false,
        totot_width = 80,
        totot_height = 30,
        totot_border = default_border_size,
        totot_hp_text = false,
        totot_mana_text = false,
        totot_mana_bar = "small",
        
        -- Focus target
        focus_enabled = true,
        focus_width = 200,
        focus_height = 46,
        focus_border = default_border_size,
        focus_mana_bar = "big",
        
        focus_buffs_enabled = true,
        focus_buffs_rows = 2,
        focus_buffs_columns = 12,
        focus_buffs_icon_size = 12,
        focus_buffs_anchor = "BOTTOMLEFT",
        
        focus_debuffs_enabled = true,
        focus_debuffs_rows = 2,
        focus_debuffs_columns = 10,
        focus_debuffs_icon_size = 28,
        focus_debuffs_anchor = "BOTTOMLEFT",
        focus_debuffs_offset_y = -24,
        
        focus_buffs2_enabled = true,
        focus_buffs2_rows = 1,
        focus_buffs2_columns = 4,
        focus_buffs2_icon_size = 32,
        focus_buffs2_anchor = "INSIDELEFT",
        focus_buffs2_group_mode = "none",
        focus_buffs2_sort_mode = "max_power",
        focus_buffs2_filter_weak = true,
        focus_buffs2_filter_normal = true,
        focus_buffs2_offset_x = default_border_size,
        
        focus_debuffs2_enabled = true,
        focus_debuffs2_rows = 1,
        focus_debuffs2_columns = 2,
        focus_debuffs2_icon_size = 38,
        focus_debuffs2_filter_weak = true,
        focus_debuffs2_group_mode = "group",
        focus_debuffs2_sort_mode = "max_power",
        focus_debuffs2_anchor = "INSIDERIGHT",
        
        -- ToF and ToToF
        tof_enabled = true,
        tof_width = 80,
        tof_height = 30,
        tof_border = default_border_size,
        tof_hp_text = false,
        tof_mana_text = false,
        tof_mana_bar = "small",
        
        totof_enabled = false,
        totof_width = 80,
        totof_height = 30,
        totof_border = default_border_size,
        totof_hp_text = false,
        totof_mana_text = false,
        totof_mana_bar = "small",
        
        -- Party members
        party_enabled = true,
        party_width = 200,
        party_height = 46,
        party_border = default_border_size,
        party_mana_bar = "big",
        
        party_buffs_enabled = true,
        party_buffs_rows = 2,
        party_buffs_columns = 8,
        party_buffs_icon_size = 12,
        party_buffs_anchor = "BOTTOMLEFT",
        
        party_debuffs_enabled = true,
        party_debuffs_rows = 2,
        party_debuffs_columns = 3,
        party_debuffs_icon_size = 28,
        party_debuffs_anchor = "RIGHT",
        
        party_buffs2_enabled = true,
        party_buffs2_rows = 3,
        party_buffs2_columns = 1,
        party_buffs2_icon_size = 24,
        party_buffs2_anchor = "LEFTTOP",
        party_buffs2_filter_weak = true,
        party_buffs2_sort_mode = "max_power",
        party_buffs2_offset_x = -8,
        
        party_debuffs2_enabled = true,
        party_debuffs2_rows = 1,
        party_debuffs2_columns = 2,
        party_debuffs2_icon_size = 38,
        party_debuffs2_filter_weak = true,
        party_debuffs2_group_mode = "group",
        party_debuffs2_sort_mode = "max_power",
        party_debuffs2_anchor = "INSIDERIGHT",
        
        -- Party pets
        party_pet_enabled = true,
        party_pet_width = 60,
        party_pet_height = 16,
        party_pet_border = default_border_size,
        party_pet_mana_bar = "none",
        party_pet_debuffs_enabled = true,
        party_pet_debuffs_rows = 1,
        party_pet_debuffs_columns = 3,
        party_pet_debuffs_icon_size = 15,
        party_pet_debuffs_anchor = "RIGHT",
        
        -- Raid members
        raid_enabled = true,
        raid_width = 60,
        raid_height = 20,
        raid_border = 2,
        raid_mana_bar = "small",
        raid_mana_text = false,
        raid_hp_text = false,
        raid_debuffs_enabled = true,
        raid_debuffs_rows = 1,
        raid_debuffs_columns = 3,
        raid_debuffs_icon_size = 16,
        raid_debuffs_anchor = "INSIDELEFT",
    }
}

if plyerclass == "PRIEST" then
    defaults.profile.shift_click_action = "Dispel Magic"
    defaults.profile.ctrl_click_action = "Abolish Disease"
elseif playerclass == "DRUID" then
    defaults.profile.shift_click_action = "Abolish Poison"
    defaults.profile.ctrl_click_action = "Remove Curse"
elseif playerclass == "MAGE" then
    defaults.profile.shift_click_action = "Remove Lesser Curse"
elseif playerclass == "PALADIN" then
    defaults.profile.shift_click_action = "Cleanse"
elseif playerclass == "WARLOCK" then
    defaults.profile.shift_click_action = "Devour Magic"
end



function TuckUF:GetDefaultConfig()
  return defaults
end

function TuckUF:ResetToDefaultConfig()
    local db = TuckUF:GetDB()
       
    for k, v in pairs(defaults) do
        db[k] = v
    end
end


local function toggle(info)
    db = core:GetDB()
    local k = info[#info]
    db[k] = nil or not db[k]
    core:ReInitialize()
end

local function copy_settings_from(info, value)
    TuckUF.db:CopyProfile(value)
    TRACE("Copied profile data from "..value)
    core:ReInitialize()
end

local function get_profile_name(info)
    return TuckUF.db:GetCurrentProfile()
end

local function set_profile_name(info, value)
    TuckUF.db:SetProfile(value)
    TRACE("Set profile to "..value)
    core:ReInitialize()
end

local function get(info)
    db = core:GetDB()
    local k = info[#info]
    return db[k] or defaults[k]
end

local function set(info,value)
    db = core:GetDB()
    local k = info[#info]
    if value and value == defaults[k] then
        value = nil
    end
    db[k] = value
    core:ReInitialize()
end

local function getnum(info)
    db = core:GetDB()
    return db[info[#info]] or 0
end

local function setnum(info,value)
    db = core:GetDB()
    local k = info[#info]
    value = tonumber(value)
    if value and value == defaults[k] then
        value = nil
    end
    db[k] = tonumber(value)
    core:ReInitialize()
end

local function geti(info)
    db = core:GetDB()
    return db[info[#info]] or 1
end

local function seti(info,value)
    db = core:GetDB()
    value = tonumber(value)
    value = value - 1
    if value < 1 then value = nil end
    db[info[#info]] = value
    core:ReInitialize()
end

local function DDAddRadioBoxes(opt, map, func, default)
    local k,v
    dewdrop:AddLine('text', default,
        'isRadio', true,
        'checked', not db[opt],
        'func', func or SetDB,
        'arg1', opt
    )
    for k,v in pairs(map) do
        dewdrop:AddLine('text', L[v] or v,
            'isRadio', true,
            'checked', db[opt] == k,
            'func', func or SetDB,
            'arg1', opt,
            'arg2', k
        )
    end
end

local function createToggle(opt, order)
    return {
        type = "toggle",
        order = order,
        get = get,
        set = toggle,
        name = L["Opt_" .. opt],
        desc = L["Desc_" .. opt],
    }
end

local function createNumInput(opt, order)
    return {
        type = "input",
        order = order,
        get = getnum,
        set = setnum,
        name = L["Opt_" .. opt],
        desc = L["Desc_" .. opt],
    }
end

local function createInput(opt, order)
    return {
        type = "input",
        order = order,
        get = get,
        set = set,
        name = L["Opt_" .. opt],
        desc = L["Desc_" .. opt],
    }
end

local function createSelection(opt, order, map)
    for _,v in ipairs(map) do
        if L[v] then v = L[v] end
    end

    return {
        type = "select",
        order = order,
        get = get,
        set = set,
        name = L["Opt_" .. opt],
        desc = L["Desc_" .. opt],
        values = map,
    }
end

local function createRange(opt, order, min, max, ispercent, step)
    return {
        type = "range",
        order = order,
        get = getnum,
        set = setnum,
        name = L["Opt_" .. opt],
        desc = L["Desc_" .. opt],
        min = min,
        max = max,
        isPercent = ispercent,
        step = step,
    }
end

local function createAuraOptions(group_name, aura_type, order)
    local map_anchors = {
        ["LEFTTOP"] = L["Opt_AnchorLeftTop"],
        ["LEFTBOTTOM"] = L["Opt_AnchorLeftBottom"],
        ["RIGHTTOP"] = L["Opt_AnchorRightTop"],
        ["RIGHTBOTTOM"] = L["Opt_AnchorRightBottom"],
        ["TOPLEFT"] = L["Opt_AnchorTopLeft"],
        ["TOPRIGHT"] = L["Opt_AnchorTopRight"],
        ["BOTTOMLEFT"] = L["Opt_AnchorBottomLeft"],
        ["BOTTOMRIGHT"] = L["Opt_AnchorBottomRight"],
        ["INSIDELEFT"] = L["Opt_AnchorInsideLeft"],
        ["INSIDERIGHT"] = L["Opt_AnchorInsideLeft"],
        ["LEFT"] = L["Opt_AnchorLeft"],
        ["RIGHT"] = L["Opt_AnchorRight"],
    }

    local map_grouping = {
        ["none"] = L["Opt_GroupingNone"],
        ["group"] = L["Opt_GroupingGroup"],
        ["name"] = L["Opt_GroupingAura"],
    }
      
    local map_sorting = {
        ["none"] = L["Opt_SortingNone"],
        ["max_duration"] = L["Opt_SortingMaxDuration"],
        ["min_duration"] = L["Opt_SortingMinDuration"],
        ["max_power"] = L["Opt_SortingMaxPower"],
        ["min_power"] = L["Opt_SortingMinPower"],
        ["max_timeleft"] = L["Opt_SortingMaxTimeleft"],
        ["min_timeleft"] = L["Opt_SortingMinTimeleft"],
    }
    
    return {
        type = "group",
        order = order,
        name = L["Opt_"..aura_type],
        desc = L["Desc_"..aura_type],
        args = {
            [group_name.."_"..aura_type.."_enabled"] = createToggle("Enabled", 1),
            [group_name.."_"..aura_type.."_icon_size"] = createRange("IconSize", 2, 12, 64, false, 2),
            [group_name.."_"..aura_type.."_icon_gap"] = createRange("IconGap", 3, 0, 4, false, 1),
            [group_name.."_"..aura_type.."_rows"] = createRange("Rows", 4, 1, 16, false, 1),
            [group_name.."_"..aura_type.."_columns"] = createRange("Columns", 5, 1, 16, false, 1),
            [group_name.."_"..aura_type.."_offset_x"] = createRange("OffsetX", 6, -64, 64, false, 2),
            [group_name.."_"..aura_type.."_offset_y"] = createRange("OffsetY", 7, -64, 64, false, 2),
            [group_name.."_"..aura_type.."_transparency"] = createRange("Transparency", 8, 0, 100, false, 2),
            [group_name.."_"..aura_type.."_anchor"] = createSelection("Anchor", 9, map_anchors),
            [group_name.."_"..aura_type.."_group_mode"] = createSelection("Grouping", 100, map_grouping),
            [group_name.."_"..aura_type.."_sort_mode"] = createSelection("Sorting", 200, map_sorting),
            [group_name.."_"..aura_type.."_filter_foreign"] = createToggle("FilterForeign", 300),
            [group_name.."_"..aura_type.."_filter_own"] = createToggle("FilterOwn", 301),
            [group_name.."_"..aura_type.."_filter_cureable"] = createToggle("FilterCureable", 302),
            [group_name.."_"..aura_type.."_filter_noncureable"] = createToggle("FilterNonCureable", 303),
            [group_name.."_"..aura_type.."_filter_weak"] = createToggle("FilterWeak", 304),
            [group_name.."_"..aura_type.."_filter_normal"] = createToggle("FilterNormal", 305),
            [group_name.."_"..aura_type.."_filter_powerful"] = createToggle("FilterPowerful", 306),
            [group_name.."_"..aura_type.."_filter_group"] = createToggle("FilterGroup", 307),
            [group_name.."_"..aura_type.."_filter_nogroup"] = createToggle("FilterNoGroup", 308),
        } -- args
    } -- buffs
end

local function createFrameOptions(group_name, order)

    local map_mana_bar = {
        ["none"] = L["Opt_ManaBarNone"],
        ["small"] = L["Opt_ManaBarSmall"],
        ["big"] = L["Opt_ManaBarBig"],
    }
    
    return {
        type = "group",
        order = order,
        name = L["Opt_"..group_name],
        desc = L["Desc_"..group_name],
        childGroups = "tab",
        args = {
            frame = {
                type = "group",
                order = 1,
                name = L["Opt_Frame"],
                desc = L["Desc_Frame"],
                args = {
                    [group_name.."_enabled"] = createToggle("Enabled", 1),
                    [group_name.."_width"] = createRange("Width", 2, 40, 400, false, 10),
                    [group_name.."_height"] = createRange("Height", 3, 10, 100, false, 5),
                    [group_name.."_border"] = createRange("Border", 4, 0, 8, false, 1),
                    [group_name.."_mana_bar"] = createSelection("ManaBar", 5, map_mana_bar),
                    [group_name.."_hp_text"] = createToggle("HPText", 6),
                    [group_name.."_hp_percent"] = createToggle("HPForcePercent", 7),
                    [group_name.."_mana_text"] = createToggle("ManaText", 8),
                    [group_name.."_combo_points"] = createToggle("ComboPoints", 9),
                } -- args
            }, -- frame
            buffs = createAuraOptions(group_name, "buffs", 2),
            buffs2 = createAuraOptions(group_name, "buffs2", 3),
            debuffs = createAuraOptions(group_name, "debuffs", 4),
            debuffs2 = createAuraOptions(group_name, "debuffs2", 5),
        }, -- args
    }
end

local options
function TuckUF:InitializeOptions()
    if not options then
        options = {
            type = "group",
            name = title,
            args = {
                player = createFrameOptions("player", 1),
                pet = createFrameOptions("pet", 2),
                party = createFrameOptions("party", 3),
                party_pet = createFrameOptions("party_pet", 4),
                target = createFrameOptions("target", 5),
                focus = createFrameOptions("focus", 6),
                raid = createFrameOptions("raid", 7),
                tot = createFrameOptions("tot", 8),
                totot = createFrameOptions("totot", 9),
                tof = createFrameOptions("tof", 10),
                totof = createFrameOptions("totof", 10),
            } -- main args
        }

        config:RegisterOptionsTable("TuckUF", {
            name = core.localizedname,
            type = "group",
            args = {
                help = {
                    type = "description",
                    name = L["AddonDescription"],
                    order = 1
                },
                click_action = createInput("click_action", 2),
                alt_click_action = createInput("alt_click_action", 3),
                shift_click_action = createInput("shift_click_action", 4),
                ctrl_click_action = createInput("ctrl_click_action", 5),
                profile_name = {
                    type = "input",
                    order = 98,
                    get = get_profile_name,
                    set = set_profile_name,
                    name = L["Opt_profile_name"],
                    desc = L["Desc_profile_name"],
                },
                copy_settings_from = {
                    type = "input",
                    order = 99,
                    get = function() return "" end,
                    set = copy_settings_from,
                    name = L["Opt_copy_settings_from"],
                    desc = L["Desc_copy_settings_from"],
                },
            },
        })
        dialog:SetDefaultSize("TuckUF", 600, 400)
        dialog:AddToBlizOptions("TuckUF", "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-ToToF", options.args.totof)
        dialog:AddToBlizOptions("TuckUF-ToToF", options.args.totof.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-ToF", options.args.tof)
        dialog:AddToBlizOptions("TuckUF-ToF", options.args.tof.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-ToToT", options.args.totot)
        dialog:AddToBlizOptions("TuckUF-ToToT", options.args.totot.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-ToT", options.args.tot)
        dialog:AddToBlizOptions("TuckUF-ToT", options.args.tot.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-Raid", options.args.raid)
        dialog:AddToBlizOptions("TuckUF-Raid", options.args.raid.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-Focus", options.args.focus)
        dialog:AddToBlizOptions("TuckUF-Focus", options.args.focus.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-Target", options.args.target)
        dialog:AddToBlizOptions("TuckUF-Target", options.args.target.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-PartyPet", options.args.party_pet)
        dialog:AddToBlizOptions("TuckUF-PartyPet", options.args.party_pet.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-Party", options.args.party)
        dialog:AddToBlizOptions("TuckUF-Party", options.args.party.name, "TuckUF")
        
        config:RegisterOptionsTable("TuckUF-Pet", options.args.pet)
        dialog:AddToBlizOptions("TuckUF-Pet", options.args.pet.name, "TuckUF")

        config:RegisterOptionsTable("TuckUF-Player", options.args.player)
        dialog:AddToBlizOptions("TuckUF-Player", options.args.player.name, "TuckUF")

        self:RegisterChatCommand("/tuf", TuckUF.ResetLayout, "TUFOVERLAYS")
    end
end

function TuckUF:ShowOptions()
    InterfaceOptionsFrame:Show()
end
