--[[
    File: TuckUnitFramesUnitFrame.lua
    Author: Tuck
    Description: This file contains all funtions that manage unit frames.

]]

local tuf_unitframe = TuckUF.unitframe
local tuf_auraframe = TuckUF.auraframe
local tuf_controlframe = TuckUF.controlframe
local tuf_network = TuckUF.network

--[[
    Libraries
]]
local IFrameFactory = IFrameFactory("1.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local TRACE = function(msg) ChatFrame1:AddMessage(msg) end


--[[
    Returs the value of the given option for the given configuration
]]
function TuckUF.unitframe:GetOption(config, option)
    local db = TuckUF.GetDB()
    return db[config.."_"..option]
end

--[[
    Initializes all subframes of the unit frame
]]
function TuckUF.unitframe:InitSubFrames(frame)
	local frameName = frame:GetName()
	
    -- get options
	frame.FrameWidth = tuf_unitframe:GetOption(frame.config, "width") or 100
	frame.FrameHeight = tuf_unitframe:GetOption(frame.config, "height") or 30
	local border_size = tuf_unitframe:GetOption(frame.config, "border") or 4
    frame.BorderSize = border_size

    -- undock mana and health bars
	frame.StatsFrame = getglobal(frameName.."_StatsFrame")
	frame.Overlays = getglobal(frameName.."_Overlays")
	frame.Background = getglobal(frameName.."_Background")
	frame.HealthBar = getglobal(frameName.."_StatsFrame_HealthBar")
	frame.HealthBar:ClearAllPoints()
	frame.ManaBar = getglobal(frameName.."_StatsFrame_ManaBar")
	frame.ManaBar:ClearAllPoints()
	
    -- set unit frame size
	tuf_unitframe:SetSize(frame)

    -- init health bar
	frame.HealthBar:SetMinMaxValues(0,100)
	frame.HealthBar:SetPoint("LEFT",frameName.."_StatsFrame", "LEFT", border_size,0)
	frame.HealthBar:SetPoint("RIGHT",frameName.."_StatsFrame", "RIGHT", -border_size,0)
	frame.HealthBar:SetPoint("TOP",frameName.."_StatsFrame", "TOP", 0, -border_size)
	frame.HealthBar:Show()
	
    -- init mana bar
	frame.ManaBar:SetMinMaxValues(0,100)
	frame.ManaBar:SetPoint("LEFT",frameName.."_StatsFrame", "LEFT", border_size,0)
	frame.ManaBar:SetPoint("RIGHT",frameName.."_StatsFrame", "RIGHT", -border_size,0)
	frame.ManaBar:SetPoint("BOTTOM",frameName.."_StatsFrame", "BOTTOM", 0, border_size)
	frame.ManaBar:SetPoint("TOP",frameName.."_StatsFrame_HealthBar", "BOTTOM", 0,0)
	frame.ManaBar:Show()

    -- init strings
	frame.HealthText = getglobal(frameName.."_Overlays_HealthText")
		frame.HealthText:SetPoint("TOPLEFT", frame.HealthBar:GetName(), "TOPLEFT", 4, 0)
		frame.HealthText:SetPoint("BOTTOMRIGHT", frame.HealthBar:GetName(), "BOTTOMRIGHT", -4, 0)
	frame.NameText = getglobal(frameName.."_Overlays_NameText")
		frame.NameText:SetPoint("TOPLEFT", frame.HealthBar:GetName(), "TOPLEFT", 4, 0)
		frame.NameText:SetPoint("BOTTOMRIGHT", frame.HealthBar:GetName(), "BOTTOMRIGHT", -4, 0)
	frame.ManaText = getglobal(frameName.."_Overlays_ManaText")
		frame.ManaText:SetPoint("TOPLEFT", frame.ManaBar:GetName(), "TOPLEFT", 4, 0)
		frame.ManaText:SetPoint("BOTTOMRIGHT", frame.ManaBar:GetName(), "BOTTOMRIGHT", -4, 0)
	frame.ClassText = getglobal(frameName.."_Overlays_ClassText")
		frame.ClassText:SetPoint("TOPLEFT", frame.ManaBar:GetName(), "TOPLEFT", 4, 0)
		frame.ClassText:SetPoint("BOTTOMRIGHT", frame.ManaBar:GetName(), "BOTTOMRIGHT", -4, 0)
		
	getglobal(frameName.."_Overlays"):SetFrameLevel(frame.ManaBar:GetFrameLevel()+10)
	
    -- backdrop for displaying the border
    frame.StatsFrame:SetBackdrop({bgFile = "Interface\\Addons\\TuckUnitFrames\\Images\\FrameBack", 
                            edgeFile = "Interface\\Addons\\TuckUnitFrames\\Images\\custom", 
                            tile = true, tileSize = 16, edgeSize = border_size, 
                            insets = { left = border_size, right = border_size, top = border_size, bottom = border_size }});
	frame.StatsFrame:SetBackdropColor(0.0,0.0,0.0,1)
	frame.StatsFrame:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
end

--[[
    Initializes all secure button functionality of the given unit frame
]]
function TuckUF.unitframe:InitSecureButton(frame)
	frame.SecureButton = frame
	
    if frame.enabled then
        frame.SecureButton:RegisterForClicks("LeftButtonDown","RightButtonDown","MiddleButtonDown","Button4Down","Button5Down")
        frame.SecureButton:EnableMouse(true)

        frame.SecureButton:SetAttribute("unit", frame.unit)
        
        local modifiers = {"", "shift-", "ctrl-", "alt-"}
        local option_names = {"click_action", "shift_click_action", "ctrl_click_action", "alt_click_action"}
        
        for i, name in ipairs(modifiers) do
            local modifier = modifiers[i]
            local option_name = option_names[i]
            local action = TuckUF.GetDB()[option_name] or ""
            
            if (action == "target") then
                frame.SecureButton:SetAttribute(modifier.."type1", "target")
            elseif (action == "focus") then
                frame.SecureButton:SetAttribute(modifier.."type1", "focus")
            elseif (action == "assist") then
                frame.SecureButton:SetAttribute(modifier.."type1", "assist")
            else
                frame.SecureButton:SetAttribute(modifier.."type1", "spell")
                frame.SecureButton:SetAttribute(modifier.."spell1", action)
            end
        end
        
        --[[
        frame.SecureButton:SetAttribute("shift-type1", "spell")
        frame.SecureButton:SetAttribute("ctrl-type1", "spell")
        frame.SecureButton:SetAttribute("alt-type1", "spell")
        frame.SecureButton:SetAttribute("shift-spell1", TuckUF.ShiftClick)
        frame.SecureButton:SetAttribute("ctrl-spell1", TuckUF.CtrlClick)
        frame.SecureButton:SetAttribute("alt-spell1", TuckUF.AltClick)
        frame.SecureButton:SetAttribute("spell1", TuckUF.Dispel1)
        ]]
        
        frame:SetAttribute("*type2", "menu")
        frame.menu = TuckUF.unitframe.OnShowMenu	
        
        RegisterUnitWatch(frame)
    else
        frame.SecureButton:EnableMouse(false)
        UnregisterUnitWatch(frame)
    end
end


--[[
    Creates a new frame
]]
function TuckUF.unitframe:CreateFrame(framename, unit, config)
    -- create frame
    local result = CreateFrame("Button", "tufFrame_"..framename, UIParent, "TuckUFUnitTemplate")
    TuckUF.unitframe:OnCreate(result, unit, config)
    return result
end

--[[
    Constructor: Performs all actions that are performed once after the frame is created
]]
function TuckUF.unitframe:OnCreate(frame, unit, config)
    -- set unit and configuration
	frame.unit = unit
    frame.old_unit = unit
    frame.config = config
    
    -- register frame with TuckUF
	tuf_controlframe.unitframes[unit] = frame
      
    -- register with click casting addons
	ClickCastFrames = ClickCastFrames or {}
	ClickCastFrames[frame] = true
    
    -- create aura frames
    if frame.aura_frames == nil then
        frame.aura_frames = {}
        frame.aura_frames["buffs"] = tuf_auraframe:CreateFrame(frame, "buffs")
        frame.aura_frames["debuffs"]  = tuf_auraframe:CreateFrame(frame, "debuffs")
        frame.aura_frames["buffs2"]  = tuf_auraframe:CreateFrame(frame, "buffs2")
        frame.aura_frames["debuffs2"]  = tuf_auraframe:CreateFrame(frame, "debuffs2")
    end
end


--[[
    Initializer: Executes all actions that performed every time the frame configuration changes
]]
function TuckUF.unitframe:OnInit(frame)    
    -- read whether the frame is enabled
    frame.enabled = tuf_unitframe:GetOption(frame.config, "enabled")
    
    -- init aura frames
    for modus, auraframe in pairs(frame.aura_frames) do
        tuf_auraframe:OnInit(auraframe)
    end
    
    -- init self
    if frame.enabled then       
        -- read some configuration
        frame.HideHPText = tuf_unitframe:GetOption(frame.config, "hp_text") == false
        frame.ForcePercentHP = tuf_unitframe:GetOption(frame.config, "hp_percent")
        frame.HideManaBar = tuf_unitframe:GetOption(frame.config, "mana_bar") == "none"
        frame.SmallManaBar = tuf_unitframe:GetOption(frame.config, "mana_bar") == "small"
        frame.HideManaText = tuf_unitframe:GetOption(frame.config, "mana_text") == false
        frame.HideClassText = tuf_unitframe:GetOption(frame.config, "mana_bar") ~= "big"
        frame.ShowComboPoints = tuf_unitframe:GetOption(frame.config, "combo_points")
        
        -- init subcomponents
        tuf_unitframe:InitSubFrames(frame)
        tuf_unitframe:InitSecureButton(frame)
    
        -- fill frame values
        tuf_unitframe:updateAll(frame)
    else      
        tuf_unitframe:InitSecureButton(frame)
        frame:Hide()
    end
end

--[[
    Changes the unit for the given frame
]]
function TuckUF.unitframe:ChangeUnit(frame, new_unit)   
    -- remove old registration
    if frame.old_unit and tuf_controlframe.unitframes[frame.old_unit] == frame then
        tuf_controlframe.unitframes[frame.old_unit] = nil
    end
    
    -- change unit
    frame.unit = new_unit
    frame.old_unit = new_unit
    
    -- reinitialize
    tuf_controlframe.unitframes[frame.unit] = frame
    tuf_unitframe:OnInit(frame)
end

--[[
    This function shows a default game menu for the corresponding unit
]]
function TuckUF.unitframe:OnShowMenu()
	local type = nil

	if self.unit == "player" then
		type = PlayerFrameDropDown
	elseif self.unit == "target" then
		type = TargetFrameDropDown
	elseif self.unit == "pet" then
		type = PetFrameDropDown
	elseif self.unit:find("party") then
        local number = string.sub(self.unit, 6)
		type = getglobal("PartyMemberFrame"..number.."DropDown")
	elseif self.unit:find("raid") then
		type = FriendsDropDown
		FriendsDropDown.displayMode = "MENU"
		FriendsDropDown.initialize = RaidFrameDropDown_Initialize
	end

	if type then
		HideDropDownMenu(1)
		type.unit = self.unit
		type.name = UnitName(self.unit)
		ToggleDropDownMenu(1, nil, type,"cursor")
		return true
	end
end

--[[

]]
function TuckUF.unitframe:OnEvent(frame, event)
	tuf_unitframe[event]()
end

--[[

]]
function TuckUF.unitframe:SetSize(frame)
    assert(frame.BorderSize ~= nil)
	frame:SetWidth(frame.FrameWidth+2*frame.BorderSize)
	frame:SetHeight(frame.FrameHeight+2*frame.BorderSize)	
	
	if frame.HideManaBar then	
		frame.HealthBar:SetHeight(frame.FrameHeight)
		frame.ManaBar:Hide()
	elseif frame.SmallManaBar then
		frame.HealthBar:SetHeight(frame.FrameHeight*(1.0-TuckUF.SmallManaBarSize))
		frame.ManaBar:SetHeight(frame.FrameHeight*(TuckUF.SmallManaBarSize))
	else
		frame.HealthBar:SetHeight(frame.FrameHeight*(1.0-TuckUF.BigManaBarSize))
		frame.ManaBar:SetHeight(frame.FrameHeight*(TuckUF.BigManaBarSize))
	end
end

--[[

]]
function TuckUF.unitframe:updateAll(frame)
	if frame and frame.enabled and UnitExists(frame.unit) then
		tuf_unitframe:updatePowerType(frame)
		tuf_unitframe:updateUnitColor(frame)
		tuf_unitframe:updateAuras(frame)
		tuf_unitframe:updateMana(frame)
		tuf_unitframe:updateHealth(frame)
		tuf_unitframe:updateNameText(frame)
		tuf_unitframe:updateClassText(frame)
        tuf_unitframe:updateBorder(frame)
	end
end

--[[

]]
function TuckUF.unitframe:ShowAll(frame)
	frame.StatsFrame:Show()
	frame.Overlays:Show()
	frame.hidden = false
end

--[[

]]
function TuckUF.unitframe:HideAll(frame)
    frame.StatsFrame:Hide()
    frame.Overlays:Hide()
    frame.hidden = true
end

--[[

]]
function TuckUF.unitframe:updatePowerType(frame)
    if not frame.enabled then return end
    
	if (UnitManaMax(frame.unit)>0) then
		if (UnitPowerType(frame.unit) == 0) then
			frame.ManaBar:SetStatusBarColor(TuckUF.ManaColor.r,TuckUF.ManaColor.g,TuckUF.ManaColor.b, TuckUF.ManaColor.a)
		elseif (UnitPowerType(frame.unit) == 1) then
			frame.ManaBar:SetStatusBarColor(TuckUF.RageColor.r,TuckUF.RageColor.g,TuckUF.RageColor.b, TuckUF.RageColor.a)
		elseif (UnitPowerType(frame.unit) == 2) then
			frame.ManaBar:SetStatusBarColor(TuckUF.EnergyColor.r,TuckUF.EnergyColor.g,TuckUF.EnergyColor.b, TuckUF.EnergyColor.a)
		elseif (UnitPowerType(frame.unit) == 3) then
			frame.ManaBar:SetStatusBarColor(TuckUF.EnergyColor.r,TuckUF.EnergyColor.g,TuckUF.EnergyColor.b, TuckUF.EnergyColor.a)
		else
			ChatFrame1:AddMessage("invalid Energy Type")
			frame.ManaBar:SetStatusBarColor(1,1,1,1)
		end
		if not frame.HideManaBar then
			frame.ManaBar:Show()
		end
	else
		frame.ManaBar:Hide()
	end
end

--[[

]]
function TuckUF.unitframe:updateUnitColor(frame)
    if not frame.enabled then return end
    
	local r, g, b = TuckUF:GetClassColor(frame.unit)
	frame.HealthBar:SetStatusBarColor(r,g,b)
end

--[[
    Updates all aura information for the given frame
]]
function TuckUF.unitframe:updateAuras(frame)
    -- if disabled, do nothing
    if not frame.enabled then return end
    
    -- update all aura icons
    for modus, auraframe in pairs(frame.aura_frames) do
        if auraframe then
            tuf_auraframe:UpdateAuras(auraframe)
        end
    end
    
    -- update border, because its color might have changed
    tuf_unitframe:updateBorder(frame)
end

--[[
    Updates the border color
]]
function TuckUF.unitframe:updateBorder(frame)
    -- reset the border
    frame.StatsFrame:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
    
    -- currently, just take the strongest special aura
    -- can be a cureable debuff or a specifically marked raid encounter aura
    local max_power = -1
    for modus, auraframe in pairs(frame.aura_frames) do
        if auraframe and auraframe.has_special_aura and auraframe.special_aura_power > max_power then
            max_power = auraframe.special_aura_power
            local c = auraframe.special_aura_color
            frame.StatsFrame:SetBackdropBorderColor(c.r, c.g, c.b, c.a)
        end
    end
    
    -- todo: color by range, line-of-sight or other brilliant ideas
end

--[[

]]
function TuckUF.unitframe:updateHealth(frame)
	if not UnitExists(frame.unit) then return end
    if not frame.enabled then return end
    
	local unit = frame.unit
	
	if ( UnitIsDead(unit) ) then
		frame.HealthBar:SetValue(0)
		if not frame.HideHPText then frame.HealthText:SetText("dead") end
		return
	elseif ( UnitIsGhost(unit) ) then
		frame.HealthBar:SetValue(0)
		if not frame.HideHPText then frame.HealthText:SetText("ghost") end
		return
	elseif ( not UnitIsConnected(unit) or maxValue == 1 ) then
		frame.HealthBar:SetValue(0)
		if not frame.HideHPText then frame.HealthText:SetText("disc") end
		return
	end
	
	local currValue,maxValue = UnitHealth(unit),UnitHealthMax(unit)
	if maxValue == 0 then
		frame.HealthText:SetText("")
		return
	end
	
	if currValue > maxValue then
		maxValue = currValue
	end
	
	local perc = currValue/maxValue * 100
	frame.HealthBar:SetValue(perc)
	--local r,g,b = TuckUF:GetHealthColor(perc)
	--frame.HealthBar:SetStatusBarColor(r, g, b)
	
	if frame.HideHPText then
		frame.HealthText:SetText("")
	elseif (frame.ForcePercentHP) then
		frame.HealthText:SetText(string.format("%.0f%%", perc))
	elseif (unit == "pet" or unit == "player" or UnitInParty(unit) or UnitInRaid(unit)) then
		frame.HealthText:SetText(currValue.."/"..maxValue)
	else
		frame.HealthText:SetText(string.format("%.0f%%", perc))
	end
end

--[[

]]
function TuckUF.unitframe:updateMana(frame)
	if not UnitExists(frame.unit) then return end
    if not frame.enabled then return end
	
    if frame.HideManaBar then 
        return 
    end
	
	local unit = frame.unit  
	
	if ( not UnitExists(unit) or UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit) ) then
		frame.ManaBar:SetValue(0)
		frame.ManaText:SetText("")
		return
	end

	local currValue, maxValue
    
    if (frame.ShowComboPoints) then
        currValue, maxValue = tuf_network:GetComboPointsByUnitID(unit), 5
    else
        currValue, maxValue = UnitMana(unit), UnitManaMax(unit)
    end
    
    if (currValue == nil) then
        currValue = 0
    end
    
	if maxValue == 0 then
		frame.ManaText:SetText("")
		frame.ManaBar:SetValue(0)
		return
	end	
	
	if currValue > maxValue then
		maxValue = currValue
	end
	
	local perc = currValue/maxValue * 100
	
	frame.ManaBar:SetValue(perc)

	if frame.HideManaText then
        frame.ManaText:SetText("")
	else
		frame.ManaText:SetText(currValue.."/"..maxValue)
	end
end

--[[

]]
function TuckUF.unitframe:updateNameText(frame)
	if not UnitExists(frame.unit) then return end
    if not frame.enabled then return end
    
	local unit = frame.unit
	frame.NameText:SetText(UnitName(unit))
end

--[[

]]
function TuckUF.unitframe:updateClassText(frame)
	if not UnitExists(frame.unit) then return end
    if not frame.enabled then return end
    
	if frame.HideClassText then 
		frame.ClassText:SetText("")
		return 
	end
	
	local unit = frame.unit
	local str = ""
	if UnitIsPlayer(unit) then
		if UnitLevel(unit) then str = UnitLevel(unit).." " end
		if UnitClass(unit) then str = str..UnitClass(unit) end
	else
		if ( classification == "rareelite" ) or (classification == "rare" ) then
			str = "rare "
		end
		
		if UnitLevel(unit)>0 then
			str = str..UnitLevel(unit)
		else
			str = "??"
		end
		
		if UnitIsPlusMob(unit) then
			str = str.."+"
		end
		
		if UnitCreatureType(unit) then
			str = str.." "..UnitCreatureType(unit)
		end
	end	
	frame.ClassText:SetText(str)
end